package main

import (
	"bufio"
	"flag"
	"fmt"
	"log"
	"os"
	"strings"
	"time"

	"github.com/araddon/dateparse"

	"github.com/olebedev/when"
	"github.com/olebedev/when/rules/common"
	"github.com/olebedev/when/rules/en"
	"github.com/olebedev/when/rules/ru"
)

var (
	timezone           = ""
	filename           = ""
	relative_time_base = ""
)

func main() {
	flag.StringVar(&timezone, "timezone", "Europe/Paris", "Timezone aka `America/Los_Angeles` formatted time-zone")
	flag.StringVar(&filename, "formats", "./zodbtools/test/testdata/tidrange-formats.txt", "Path of tidrange-formats.txt")
	flag.StringVar(&relative_time_base, "relative-time-base", "2009-08-30T19:20:00Z", "Consider relative time as relative to this RFC3339 formatted time")
	flag.Parse()

	if timezone != "" {
		loc, err := time.LoadLocation(timezone)
		if err != nil {
			panic(err.Error())
		}
		time.Local = loc
	}

	relative_time_base_time := time.Now()
	if relative_time_base != "" {
		t, err := time.Parse(time.RFC3339, relative_time_base)
		if err != nil {
			panic(err.Error())
		}
		relative_time_base_time = t.In(time.Local)
	}

	// initialize when library
	w := when.New(nil)
	w.Add(en.All...)
	w.Add(ru.All...)
	w.Add(common.All...)

	// github.com/araddon/dateparse does not like '1.day.ago' style, so we preprocess inputs to remove . and _
	preprocessor := strings.NewReplacer(".", " ", "_", " ")

	file, err := os.Open(filename)
	if err != nil {
		log.Fatal(err)
	}
	defer file.Close()

	scanner := bufio.NewScanner(file)
	for scanner.Scan() {
		line := scanner.Text()
		// skip comments or empty lines
		if strings.Index(line, "#") != 0 && line != "" {
			parts := strings.SplitN(line, " ", 3)
			reference := parts[0]
			// reference can be in RFC3339Nano or RFC3339 format, but
			// different implementations does not produce same output
			// ie. python outputs 1985-04-12T23:20:50.5200Z
			//     golang outputs 1985-04-12T23:20:50.50Z
			// so we reformat the date.
			reference_time, err := time.Parse(time.RFC3339Nano, reference)
			if err != nil {
				panic(err.Error())
			}
			reference = reference_time.Format(time.RFC3339Nano)

			timespec := parts[2]

			preprocessed_timespec := timespec
			if strings.Contains(timespec, "ago") {
				preprocessed_timespec = preprocessor.Replace(timespec)
			}
			t, err := dateparse.ParseAny(preprocessed_timespec)
			if err != nil {
				r, err := w.Parse(preprocessed_timespec, relative_time_base_time)
				if err != nil {
					panic(err.Error())
				}
				if r == nil {
					fmt.Println("ERROR no match")
				} else {
					t = r.Time
				}
			}

			parsed_time := t.UTC().Format(time.RFC3339Nano)
			if parsed_time != reference {
				fmt.Println(
					"ERROR timespec:", timespec,
					/* "( preprocessed as", preprocessed_timespec, ")", */
					"expected time:", reference,
					"parsed time:", parsed_time,
				)
			}

		}
	}
}
