from __future__ import print_function
import argparse
import datetime
import os
import time

import dateparser
import pytz
import freezegun

# this is golang's time.RFC3339
rfc3339_format = "%Y-%m-%dT%H:%M:%SZ"
# this is golang's time.RFC3339Nano
rfc3339nano_format = "%Y-%m-%dT%H:%M:%S.%fZ"

parser = argparse.ArgumentParser(
    description="Prototype script for tidrange parsing")
parser.add_argument(
    "--formats",
    default="./zodbtools/test/testdata/tidrange-formats.txt",
    help="Path of tidrange-formats.txt",
)
parser.add_argument(
    "--timezone",
    default="Europe/Paris",
    help="Timezone aka `America/Los_Angeles` formatted time-zone",
)
parser.add_argument(
    "--relative-time-base",
    default="2009-08-30T19:20:00Z",
    help="Consider relative time as relative to this RFC3339 formatted time",
)
args = parser.parse_args()

dateparser_settings = {
    "TO_TIMEZONE": "UTC",
    "RETURN_AS_TIMEZONE_AWARE": True,
   # "STRICT_PARSING": True
}

# process timezone
os.environ["TZ"] = args.timezone
time.tzset()

# process relative time
if args.relative_time_base and args.relative_time_base != "now":
    # dateparsers' has a RELATIVE_BASE option for this, but this does not work well,
    # this triggers some TypeError internally, so use freezetime to change
    # time base.
    freezegun.freeze_time(
        datetime.datetime.strptime(
            args.relative_time_base, rfc3339_format).replace(
                tzinfo=pytz.utc  # the Z in our format means UTC
            ).astimezone(pytz.timezone(args.timezone))).start()

with open(args.formats, "r") as f:
    for line in f:
        line = line.strip()
        # skip comments or empty lines
        if not line or line.startswith("#"):
            continue

        reference, _, timespec = line.split(" ", 2)
        # reference can be in RFC3339Nano or RFC3339 format, but
	    # different implementations does not produce same output
		# ie. python outputs 1985-04-12T23:20:50.5200Z
		#     golang outputs 1985-04-12T23:20:50.50Z
		# so we reformat the date.
        try:
            reference_time = datetime.datetime.strptime(reference, rfc3339nano_format)
        except ValueError:
            reference_time = datetime.datetime.strptime(reference, rfc3339_format)
        reference = reference_time.strftime(rfc3339nano_format)

        preprocessed_timespec = timespec
        if "ago" in timespec:
            preprocessed_timespec = timespec.replace(".", " ").replace(
                "_", " ")

        parsed_time = dateparser.parse(
            preprocessed_timespec, settings=dateparser_settings)
        if parsed_time:
            parsed_time = parsed_time.astimezone(
                pytz.utc).strftime(rfc3339nano_format)

        if reference != parsed_time:
            print(
                "ERROR timespec:",
                timespec,
                # "( preprocessed as", preprocessed_timespec, ")",
                "expected time:",
                reference,
                "parsed time:",
                parsed_time
            )
